import type { JwaEncryptionAlgorithm } from '../jwa/alg';
import type { JwkJson } from './Jwk';
import { KeyType } from '../../KeyType';
import { JwaCurve, JwaKeyType } from '../jwa';
import { JwaSignatureAlgorithm } from '../jwa/alg';
import { Jwk } from './Jwk';
export declare class P521Jwk extends Jwk {
    static readonly supportedEncryptionAlgorithms: JwaEncryptionAlgorithm[];
    static readonly supportedSignatureAlgorithms: JwaSignatureAlgorithm[];
    static readonly keyType = KeyType.P521;
    private readonly affinePoint;
    constructor({ x, y }: {
        x: string | Uint8Array;
        y: string | Uint8Array;
    });
    get kty(): JwaKeyType.EC;
    get crv(): JwaCurve.P521;
    get keyType(): KeyType;
    get supportedEncryptionAlgorithms(): JwaEncryptionAlgorithm[];
    get supportedSignatureAlgorithms(): JwaSignatureAlgorithm[];
    get x(): string;
    get y(): string;
    /**
     * Returns the uncompressed public key of the P-521 JWK.
     */
    get publicKey(): Uint8Array;
    /**
     * Returns the compressed public key of the P-521 JWK.
     */
    get publicKeyCompressed(): Uint8Array;
    toJson(): P521JwkJson;
    static fromJson(jwk: JwkJson): P521Jwk;
    static fromPublicKey(publicKey: Uint8Array): P521Jwk;
}
export interface P521JwkJson extends JwkJson {
    kty: JwaKeyType.EC;
    crv: JwaCurve.P521;
    x: string;
    y: string;
    use?: 'sig' | 'enc';
}
export declare function isValidP521JwkPublicKey(jwk: JwkJson): jwk is P521JwkJson;
